SystemOrganization addCategory: #'OmniST-Core-Misc'!
SystemOrganization addCategory: #'OmniST-Core-Compiler'!
SystemOrganization addCategory: #'OmniST-Core-Reflection'!

!TPureBehavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/8/2011 14:58'!
flushOmniMethods
	| copy |
	self methodDictionary keysDo: [ :each |
		each isOmniSelector ifTrue: [
			copy := copy ifNil: [ self methodDictionary copy ].
			copy removeDangerouslyKey: each ifAbsent: [] ] ].
	copy ifNotNil: [ self methodDictionary become: copy ]! !

!TPureBehavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/11/2011 16:44'!
selectors
	"Answer a Set of all the message selectors specified in the receiver's 
	method dictionary."

	^ self methodDict keys reject: [ :each | each isOmniSelector ]! !

!TPureBehavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/11/2011 16:44'!
selectorsAndMethodsDo: aBlock
	"Evaluate selectorBlock for all the message selectors in my method dictionary."

	^ self methodDict keysAndValuesDo: [ :selector :method |
		selector isOmniSelector 
			ifFalse: [ aBlock value: selector value: method ] ]! !

!TPureBehavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/11/2011 16:44'!
selectorsDo: selectorBlock
	"Evaluate selectorBlock for all the message selectors in my method dictionary."

	^ self methodDict keysDo: [ :selector |
		selector isOmniSelector 
			ifFalse: [ selectorBlock value: selector ] ]! !

!Process methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/11/2011 22:50'!
omniDomain
	<omniDoNotTransform>
	^ omniDomain! !

!Process methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/11/2011 22:50'!
omniDomain: aDomain
	<omniDoNotTransform>
	omniDomain := aDomain! !

!Exception methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:35'!
resumeUnchecked: resumptionValue
	"Return resumptionValue as the value of #signal, unless this was called after an #outer message, then return resumptionValue as the value of #outer."
	<omniDoNotTransform>
	| ctxt |
	outerContext 
		ifNil: [
			signalContext return: resumptionValue ] 
		ifNotNil: [
			ctxt := outerContext.
			outerContext := ctxt tempAt: 1. "prevOuterContext in #outer"
			ctxt return: resumptionValue ]
! !

!BlockClosure methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/15/2011 15:07'!
omni
	^ OstDomain new evaluateInContext: self.! !

!BlockClosure methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/8/2011 16:01'!
omni: aDomain
	^ aDomain evaluateInContext: self.! !

!ProcessorScheduler methodsFor: '*OmniST-Core-Roar-Compatibility' stamp: 'StefanMarr 11/6/2011 14:46'!
thisProcess
	"Added for RVM compatibility already.
	 Just to make sure that I can switch easily."
	^ self activeProcess! !

RBSemanticAnnotator subclass: #OstTransformer
	instanceVariableNames: 'doFullTransformation doLocalTransformation numOfOmniTemps'
	classVariableNames: 'RequestExecutionSelectors RequestSuperExecutionSelectors'
	poolDictionaries: ''
	category: 'OmniST-Core-Compiler'!

!OstTransformer commentStamp: '<historical>' prior: 0!
OmniTransformer iterates over the AST of a method to transform it.
The goal is to delegate all read, write, and message-send operations via a domain object.!

!OstTransformer class methodsFor: 'initialization' stamp: 'StefanMarr 3/20/2012 17:48'!
generateRequestExecutionSelectors
	| sel max |
	max := 15.
	sel := #requestExecutionOf:.
	RequestExecutionSelectors := Array new: max.
	1 to: max do: [:i |
		RequestExecutionSelectors at: i put: sel.
		sel := (sel, #with:) asSymbol.
	].

	sel := #requestExecutionOf:lookup:.
	RequestSuperExecutionSelectors := Array new: max.
	1 to: max do: [:i |
		RequestSuperExecutionSelectors at: i put: sel.
		sel := (sel, #with:) asSymbol.
	].! !

!OstTransformer class methodsFor: 'initialization' stamp: 'StefanMarr 12/8/2011 00:58'!
initialize
	self generateRequestExecutionSelectors.! !

!OstTransformer methodsFor: 'visitor-double dispatching' stamp: 'StefanMarr 12/7/2011 20:49'!
acceptAssignmentNode: anAssignmentNode
	super acceptAssignmentNode: anAssignmentNode.
	
	self doTransformation ifFalse: [^self].
	
	(anAssignmentNode variable isInstanceBinding or:
	[anAssignmentNode variable isLiteralBinding "or: REMOTE NOT YET SUPPORTED
	[anAssignmentNode variable isRemote]"]) ifTrue: [
		| replacement |
		replacement := self createOmniAssignment: anAssignmentNode.
		anAssignmentNode parent replaceNode: anAssignmentNode withNode: replacement.
	]! !

!OstTransformer methodsFor: 'visitor-double dispatching' stamp: 'StefanMarr 12/8/2011 15:39'!
acceptMessageNode: aMessageNode
	| doLocalTransformationOld replacement |
	doLocalTransformationOld := doLocalTransformation.
	
	doLocalTransformation := doLocalTransformation or: [self isOmniEntryMessage: aMessageNode selector].

	(self doTransformation and: [aMessageNode isInlined not])
		ifTrue:  [self  transformMessageNode: aMessageNode.]
		ifFalse: [super acceptMessageNode:    aMessageNode.].

	doLocalTransformation := doLocalTransformationOld.! !

!OstTransformer methodsFor: 'visitor-dispatching' stamp: 'StefanMarr 12/7/2011 20:55'!
acceptMethodNode: aMethodNode
	super acceptMethodNode: aMethodNode.
	doFullTransformation ifTrue: [
		aMethodNode selector: (aMethodNode selector asOmniSelector)
	].! !

!OstTransformer methodsFor: 'initialize-release' stamp: 'StefanMarr 1/2/2012 18:38'!
acceptOmniEntryNode: aMessageNode
	(self omniEntryBlockIsReceiver: aMessageNode selector)
		ifTrue: [
			"Then we do not do a full processing, but are skipping the arguments"
			(aMessageNode isCascaded not or: [aMessageNode isFirstCascaded]) 
					ifTrue: [self visitNode: aMessageNode receiver].]
		ifFalse:[
			"If the block to be omnified is the argument however, we do transform it."
			aMessageNode arguments do: [:each | self visitNode: each]]! !

!OstTransformer methodsFor: 'visitor-dispatching' stamp: 'StefanMarr 12/8/2011 00:56'!
acceptVariableNode: aVariableNode
	| replacement |
	self doTransformation ifFalse: [^self].
	
	super acceptVariableNode: aVariableNode.
	
	"Not every variable node we are going to encounter is a read.
	 Make sure it is not the variable node of a parent assignment node.
	 Furthermore, we do not need to take care of temp assignments.
	 Only globals, instances, and perhaps remote onces of a closure."
	(aVariableNode isRead
	 and: [aVariableNode isLiteralBinding   or: 
		   [aVariableNode isInstanceBinding "or: REMOTES NOT SUPPORTED at the moment
			[aVariableNode isRemote]"]])
		ifFalse: [^self].
	
	"Now we are sure that we have to transform a variable read".
	replacement := self createOmniReadNode: aVariableNode.
	aVariableNode parent replaceNode: aVariableNode withNode: replacement. 
! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/15/2011 14:13'!
applyTransformations: methodNode inScopeOf: aClass
	"We have two cases:
	   Either it is a normal method, and we do only the omni blocks,
	   or it is a fully omni method and we transform everything.
	
	 doFullTransformation indicates that we do not only local adaptations.
	"
	| classScope |
	(self pragmasForbidTransformation: methodNode)
		ifTrue: [^self].
	
	classScope := RBVariableScope
							owner: (RBLiteralScope 	owner: RBRootScope new class: aClass)
							class: aClass.
	
	self start: methodNode scope: classScope.! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/15/2011 14:35'!
createObtainCurrentOmniDomainNode
	| msgSendNode  |
	msgSendNode := RBMessageNode receiver: (RBLiteralValueNode value: OstDomain)
									   selector: #currentDomain.
	msgSendNode beOmniNode.
	^ msgSendNode.! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 7/18/2012 21:47'!
createOmniAssignment: anAssignmentNode
	| var |
	var := anAssignmentNode variable.
	var isInstanceBinding ifTrue: [
		^ self createWriteMessageSendFor: var with: anAssignmentNode value.
	]
	ifFalse: [
		| globalAssoc sendArgs |
		var isLiteralBinding ifFalse: [self halt: 'I guess we still need to implement something, e.g., remote vars.'].
		
		^ self createWriteGlobalMessageSendFor: var with: anAssignmentNode value.
	].! !

!OstTransformer methodsFor: 'visitor-double dispatching' stamp: 'StefanMarr 12/15/2011 14:36'!
createOmniDomainRefNode
	"Return a node that will resolve at runtime to the omni domain object,
	 or in case the class does not have a slot called omniDomain, resolve to
	 the nil object"
	| omniIdx omniRef |
	omniIdx := OstDomain domainIndexFromClass: scope theClass.
	omniIdx = 0
		ifTrue: [
			"The class does not have a omniDomain slot, so we look it up"
			omniRef := self createObtainCurrentOmniDomainNode.
		]
		ifFalse:[
			"This class got a omniDomain slot"
			omniRef := RBVariableNode named: #omniDomain.
			omniRef variableBinding: (scope lookup: #omniDomain).
		].
	
	^ omniRef! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/15/2011 14:25'!
createOmniMessageNode: aMessageNode
	| omniSel |

	omniSel := aMessageNode selector asOmniSelector.
	
	^ self createRequestExecutionNodeOn: aMessageNode receiver
										 with: omniSel
										  and: aMessageNode arguments.
! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/9/2011 15:50'!
createOmniReadNode: aVariableNode
	| selfRef omniDomainRef instIdx sendArgs |
	
	aVariableNode isInstanceBinding ifTrue: [
		^ self createReadMessageSendFor: aVariableNode.
	]
	ifFalse: [
		aVariableNode isLiteralBinding
			ifFalse: [self halt: 'I guess we still need to implement something, e.g., remote vars.'].
		^ self createReadGlobalMessageSendFor: aVariableNode.
	].
	! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 5/1/2012 21:34'!
createReadGlobalMessageSendFor: aVariableNode
	| globalAssoc args sendNode |

	globalAssoc := RBLiteralValueNode value: aVariableNode variableBinding binding.
		
	args := OrderedCollection new: 1.
	args add: globalAssoc.
		
	sendNode := RBMessageNode receiver: self createObtainCurrentOmniDomainNode
		                          selector: #readLiteral:
		                          arguments: args.
	sendNode beOmniNode.
	^ sendNode! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/8/2011 01:21'!
createReadMessageSendFor: aVariableNode
	| selfRef omniDomainRef instIdx sendArgs readNode |
	selfRef := self createSelfRefNode.
	omniDomainRef := self createOmniDomainRefNode.
	
	instIdx  := RBLiteralValueNode value: (aVariableNode variableBinding index).
	sendArgs := OrderedCollection new: 2.
	sendArgs add: instIdx.
	sendArgs add: selfRef.
	readNode := RBMessageNode receiver: omniDomainRef selector: #readField:of: arguments: sendArgs.
	readNode beOmniNode.
	^ readNode
! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 3/20/2012 17:38'!
createRequestExecutionNodeOn: receiver with: selector and: arguments
	| msgSendNode reqExecSelector msgArguments numArgs isSuperSend newRcvr |
	
	"REM: we send to self, to avoid more complex rewrites
	  #requestExecutionOf:with:...   or   #requestExecutionOf:lookup:with:..."
	numArgs := selector numArgs.

	isSuperSend := receiver isVariable
						ifTrue:  [receiver variableBinding isSuperBinding]
						ifFalse: [false].
	newRcvr := isSuperSend ifTrue: [self createSelfRefNode]
								ifFalse:[receiver].

	reqExecSelector := self requestExecutionSelectorFor: numArgs withSuper: isSuperSend.
	
	"One extra argument for the class where the lookup starts."
	isSuperSend ifTrue: [ numArgs := numArgs + 1. ].
	
	"And another argument for the selector"
	numArgs := numArgs + 1.
	
	msgArguments := OrderedCollection new: numArgs.
	msgArguments add: (RBLiteralValueNode value: selector).
	isSuperSend ifTrue: [
		msgArguments add: (RBLiteralValueNode value: scope theClass superclass).
	].
	msgArguments addAll: arguments.
	
	msgSendNode := RBMessageNode receiver:  newRcvr
								      selector:  reqExecSelector
								      arguments: msgArguments.
	msgSendNode beOmniNode.
	^ msgSendNode! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/8/2011 01:11'!
createSelfRefNode
	| selfRef |
	selfRef := RBVariableNode named: 'self'.
	selfRef variableBinding: (scope lookup: 'self').
	^ selfRef! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 7/18/2012 21:45'!
createWriteGlobalMessageSendFor: aVariableNode with: valueNode
	| globalAssoc args sendNode |

	globalAssoc := RBLiteralValueNode value: aVariableNode variableBinding binding.
		
	args := OrderedCollection new: 2.
	args add: valueNode.
	args add: globalAssoc.
		
	sendNode := RBMessageNode receiver: self createObtainCurrentOmniDomainNode
		                          selector: #write:toLiteral:
		                          arguments: args.
	sendNode beOmniNode.
	^ sendNode! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/8/2011 21:57'!
createWriteMessageSendFor: aVarNode with: aValue
		| selfRef omniDomainRef instIdx sendArgs sendNode |
		selfRef := self createSelfRefNode.
		omniDomainRef := self createOmniDomainRefNode.
		
		instIdx := RBLiteralValueNode value: (aVarNode variableBinding index).
		
		sendArgs := OrderedCollection new: 3.
		sendArgs add: aValue.
		sendArgs add: instIdx.
		sendArgs add: selfRef.
		
		sendNode := RBMessageNode receiver: omniDomainRef
									   selector: #write:toField:of:
									   arguments: sendArgs.
		sendNode beOmniNode.
		^ sendNode
! !

!OstTransformer methodsFor: 'testing' stamp: 'StefanMarr 12/7/2011 20:35'!
doTransformation
	^ doFullTransformation or: [doLocalTransformation]! !

!OstTransformer methodsFor: 'accessing' stamp: 'StefanMarr 12/7/2011 20:35'!
fullTransformation: aBool
	doFullTransformation := aBool! !

!OstTransformer methodsFor: 'initialize-release' stamp: 'StefanMarr 12/9/2011 15:29'!
initialize
	super initialize.
	doFullTransformation  := false.
	doLocalTransformation := false.
	numOfOmniTemps := 0.
! !

!OstTransformer methodsFor: 'visitor-double dispatching' stamp: 'StefanMarr 1/3/2012 11:22'!
isOmniEntryMessage: aSelector
	^ aSelector = #omni            or: [
	  aSelector = #omni:           or: [
	  aSelector = #atomic          or: [
	  aSelector = #evaluateAsync:  or: [
	  aSelector = #newWithInitializer:
	]]]].! !

!OstTransformer methodsFor: 'testing' stamp: 'StefanMarr 12/8/2011 15:44'!
isOmniTransformationPragma: aSelector
	^ aSelector = #omniDoNotTransform or: 
	 [aSelector = #omniReplacement:]! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 12/9/2011 15:29'!
nextOmniTempIdx
	"Creates a sequence of unique id numbers for a method transformation to name the temporary variables"
	numOfOmniTemps := numOfOmniTemps + 1.
	^ numOfOmniTemps! !

!OstTransformer methodsFor: 'initialize-release' stamp: 'StefanMarr 1/3/2012 11:21'!
omniEntryBlockIsReceiver: aSelector
	(aSelector = #omni            or: [
	 aSelector = #omni:           or: [
	 aSelector = #atomic]])
		ifTrue: [^true].
		
	(aSelector = #evaluateAsync:  or: [
	 aSelector = #newWithInitializer:])
		ifTrue: [^ false].
	

	"Else"
	'This selector is not currently known' halt.! !

!OstTransformer methodsFor: 'testing' stamp: 'StefanMarr 12/8/2011 15:42'!
pragmasForbidTransformation: methodNode
	"Some of our pragmas restrict the boundaries of the Omni execution layer.
	 They enable to break out into the implementation level."
	
	methodNode pragmas do: [:each |
		(self isOmniTransformationPragma: each selector)
			ifTrue: [^true].
	].
	^false.! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 1/2/2012 18:07'!
requestExecutionSelectorFor: nArgs withSuper: superSend
	(nArgs >= 0 and: [nArgs < RequestSuperExecutionSelectors size])
		ifTrue: [
			superSend
				ifTrue: [^ RequestSuperExecutionSelectors at: nArgs + 1.]
				ifFalse:[^ RequestExecutionSelectors at: nArgs + 1.]
		]
		ifFalse: [
			self halt: 'This is currently not implemented!! ',
			           'Need either to extend this table/array, ',
			           'or make a distinction to use the other primitive (84) which accepts an array.'.
		].
	^ nil! !

!OstTransformer methodsFor: 'transformation' stamp: 'StefanMarr 1/2/2012 18:37'!
transformMessageNode: aMessageNode
	(doFullTransformation or: [(self isOmniEntryMessage: aMessageNode selector) not])
		ifTrue: [ super acceptMessageNode: aMessageNode. ]
		ifFalse:[
			"This is the case if it is the omni entry node."
			self acceptOmniEntryNode: aMessageNode.
		].

	((self isOmniEntryMessage: aMessageNode selector) or: [
	 aMessageNode isInlined or: [
	 aMessageNode isOmniNode or: [
	 aMessageNode receiver isLiteral ]]]) "REM: added optimization for literal nodes here, we do not omnify those"
		ifFalse: [
			| replacement |
			replacement := self createOmniMessageNode: aMessageNode.
			aMessageNode parent replaceNode: aMessageNode withNode: replacement.
		].! !

!Behavior methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 18:10'!
basicNew
	"Primitive. Answer an instance of the receiver (which is a class) with no 
	indexable variables. Fail if the class is indexable. Essential. See Object 
	documentation whatIsAPrimitive."

	<primitive: 70>
	<omniReplacement: #omniBasicNew>
	self isVariable ifTrue: [ ^ self basicNew: 0 ].
	"space must be low"
	OutOfMemory signal.
	^ self basicNew  "retry if user proceeds"! !

!Behavior methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 18:11'!
basicNew: sizeRequested 
	"Primitive. Answer an instance of this class with the number
	of indexable variables specified by the argument, sizeRequested.
	Fail if this class is not indexable or if the argument is not a
	positive Integer, or if there is not enough memory available. 
	Essential. See Object documentation whatIsAPrimitive."

	<primitive: 71>
	<omniReplacement: #omniBasicNew:>
	self isVariable ifFalse:
		[self error: self printString, ' cannot have variable sized instances'].
	(sizeRequested isInteger and: [sizeRequested >= 0]) ifTrue:
		["arg okay; space must be low."
		OutOfMemory signal.
		^ self basicNew: sizeRequested  "retry if user proceeds"].
	self primitiveFailed! !

!Behavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/8/2011 14:55'!
flushOmniMethods
	| copy |
	self methodDictionary keysDo: [ :each |
		each isOmniSelector ifTrue: [
			copy := copy ifNil: [ self methodDictionary copy ].
			copy removeDangerouslyKey: each ifAbsent: [] ] ].
	copy ifNotNil: [ self methodDictionary become: copy ]! !

!Behavior methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/11/2011 18:10'!
omniBasicNew
	| obj |
	<omniDoNotTransform>
	obj := self basicNew.
	OstDomain initializeDomainForNewObject: obj of: self.
	^ obj! !

!Behavior methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/11/2011 18:11'!
omniBasicNew: sizeRequested 
	"This is the omni-fied implementation.
	 The original basicNew: is now rawBasicNew:."
	<omniDoNotTransform>
	
	| obj |
	obj := self basicNew: sizeRequested.
	OstDomain initializeDomainForNewObject: obj of: self.
	^ obj! !

!Behavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/9/2011 17:08'!
omniMethods
	^ Array streamContents: [ :stream |
		self methodDictionary keysAndValuesDo: [ :key :value |
			key isOmniSelector 
				ifTrue: [ stream nextPut: value ] ] ]! !

!Behavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/9/2011 17:05'!
selectors
	"Answer a Set of all the message selectors specified in the receiver's 
	method dictionary."

	^ self methodDict keys reject: [ :each | each isOmniSelector ]! !

!Behavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/9/2011 17:05'!
selectorsAndMethodsDo: aBlock
	"Evaluate selectorBlock for all the message selectors in my method dictionary."

	^ self methodDict keysAndValuesDo: [ :selector :method |
		selector isOmniSelector 
			ifFalse: [ aBlock value: selector value: method ] ]! !

!Behavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/9/2011 17:05'!
selectorsDo: selectorBlock
	"Evaluate selectorBlock for all the message selectors in my method dictionary."

	^ self methodDict keysDo: [ :selector |
		selector isOmniSelector 
			ifFalse: [ selectorBlock value: selector ] ]! !

!ByteArray methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 18:47'!
replaceFrom: start to: stop with: replacement startingAt: repStart
	"Primitive. This destructively replaces elements from start to stop in the receiver starting at index, repStart, in the collection, replacement. Answer the receiver. Range checks are performed in the primitive only. Optional. See Object documentation whatIsAPrimitive."
	
	<primitive: 105>
	<omniReplacement: #omniReplaceFrom:to:with:startingAt:>
	super replaceFrom: start to: stop with: replacement startingAt: repStart! !

!RBMethodNode methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/3/2011 22:26'!
sourceText
	^source! !

!MethodNode methodsFor: '*OmniST-Core-Compiler-code generation' stamp: 'StefanMarr 11/28/2011 14:36'!
generate: trailer ifQuick: methodBlock
	| v |
	(primitive = 0 and: [arguments size = 0 and: [block isQuick]])
		ifFalse: [^ self].	
	v := block code.
	v < 0
		ifTrue: [^ self].
	v = LdSelf
		ifTrue: [^ self].
	(v between: LdTrue and: LdMinus1 + 3)
		ifTrue: [^ methodBlock value: (CompiledMethod toReturnConstant: v - LdSelf trailerBytes: trailer)].
	v < ((CodeBases at: LdInstType) + (CodeLimits at: LdInstType))
		ifTrue: [^ self].
	^ self.
	! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/8/2011 00:02'!
beOmniNode
	isOmniNode := true.! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler-inline' stamp: 'JorgeRessia 11/25/2009 13:33'!
isInlineAndOr
	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(self selectorParts allSatisfy: [ :each | #(and: or:) includes: each value ]) 
		ifFalse: [^ false].
	(self arguments allSatisfy: [ :each | each isBlock ])  ifFalse: [^ false].
	(self arguments allSatisfy: [ :each | each arguments isEmpty ])
			ifFalse: [
				self notify: 'and: (or:) takes zero-arg block'.
				^ false ].
	^ true! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler-inline' stamp: 'jorgeRessia 11/20/2009 16:40'!
isInlineCase

	self isCascaded ifTrue: [^ false].
	(#(caseOf: caseOf:otherwise:) includes: self selector) ifFalse: [^ false].
	self arguments size = 2 ifTrue: [  "otherwise block"
		self arguments last isBlock ifFalse: [^ false]].
	self arguments first isArray ifFalse: [^ false].
	self arguments first statements do: [:assoc |
		(assoc isMessage and: [assoc selector == #->]) ifFalse: [^ false].
		assoc receiver isBlock ifFalse: [^ false].
		assoc receiver arguments isEmpty ifFalse: [self notify: 'caseOf: takes zero-arg blocks'. ^ false].
		assoc arguments first isBlock ifFalse: [^ false].
		assoc arguments first arguments isEmpty ifFalse: [self notify: 'caseOf: takes zero-arg blocks'].
	].
	^ true! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler-inline' stamp: 'jorgeRessia 11/20/2009 16:40'!
isInlineIf

	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(#(ifTrue: ifFalse: ifTrue:ifFalse: ifFalse:ifTrue:) includes: self selector) ifFalse: [^ false].
	self arguments do: [:node | node isBlock ifFalse: [^ false]].
	self arguments do: [:block |
		block arguments isEmpty ifFalse: [
			self notify: 'ifTrue:ifFalse: takes zero-arg blocks'.
			^ false
		]
	].
	^ true! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler-inline' stamp: 'MarcusDenker 8/21/2010 13:40'!
isInlineIfNil

	| assertNone assertOneOrNone |
	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(#(ifNil: ifNotNil: ifNil:ifNotNil: ifNotNil:ifNil:) includes: self selector) ifFalse: [^ false].
	self arguments do: [:node | node isBlock ifFalse: [^ false]].
	assertNone := [:block | 
		block arguments isEmpty ifFalse: [self notify: 'ifNil: takes zero-arg block'. ^ false]
	].
	assertOneOrNone := [:block | 
		block arguments size > 1 ifTrue: [self notify: 'ifNotNil: takes zero- or one-arg block'. ^ false]
	].
	self selector caseOf: {
		[#ifNil:] 		 	-> [assertNone value: self arguments first].
		[#ifNil:ifNotNil:] 	-> [assertNone value: self arguments first. 
							   assertOneOrNone value: self arguments last].
		[#ifNotNil:] 		  	-> [assertOneOrNone value: self arguments first].
		[#ifNotNilDo:] 		  	-> [assertOneOrNone value: self arguments first].
		[#ifNotNil:ifNil:] 	-> [assertOneOrNone value: self arguments first. 
							   assertNone value: self arguments last]
	}.
	^ true! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler-inline' stamp: 'MarcusDenker 9/3/2010 15:03'!
isInlineToDo

	| block step |
	self receiver isBlock ifTrue: [^ false].
	self isCascaded ifTrue: [^ false].
	(#(to:do: to:by:do:) includes: self selector) ifFalse: [^ false].
	self arguments first isBlock ifTrue: [^ false].
	block := self arguments last.
	block isBlock ifFalse: [^ false].
	block arguments size = 1 ifFalse: [
		self notify: 'to:do: block must take one arg'. ^ false].
	(RBParseTreeSearcher new
		matches: block arguments first name , ' := `@object' do: [:n :a | true];
		executeTree: block initialAnswer: false) ifTrue: [^ false].
	self arguments size = 3 "to:by:do:" ifTrue: [
		step := self arguments second.
		step isLiteral ifFalse: [^ false].
		step value = 0 ifTrue: [self notify: 'by: step can''t be zero'. ^ false].
	].
	^ true! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler-inline' stamp: 'jorgeRessia 11/20/2009 16:40'!
isInlineWhile

	self isCascaded ifTrue: [^ false].
	(#(whileFalse: whileTrue: whileFalse whileTrue) includes: self selector) ifFalse: [^ false].
	self receiver isBlock ifFalse: [^ false].
	self receiver arguments isEmpty ifFalse: [self notify: 'while receiver block must have no arguments'. ^ false].
	self arguments isEmpty ifFalse: [
		self arguments first isBlock ifFalse: [^ false].
		self arguments first arguments isEmpty ifFalse: [self notify: 'while takes a zero-arg block as its argument'. ^ false].
	].
	^ true! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler-inline' stamp: 'StefanMarr 12/3/2011 14:22'!
isInlined
	^ self isInlineAndOr or:
	 [self isInlineCase  or:
	 [self isInlineIf    or:
	 [self isInlineIfNil or:
	 [self isInlineToDo  or:
	  self isInlineWhile    ]]]]! !

!RBMessageNode methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/8/2011 00:06'!
isOmniNode
	^ isOmniNode = true "with this, we do not need to change initialize"! !

!TraitBehavior methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/11/2011 16:49'!
selectorsAndMethodsDo: aBlock
	"Evaluate selectorBlock for all the message selectors in my method dictionary."

	^ self methodDict keysAndValuesDo: [ :selector :method |
		selector isOmniSelector 
			ifFalse: [ aBlock value: selector value: method ] ]! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/9/2012 17:59'!
domainForNewObjects
	^ nil! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/8/2011 21:53'!
evaluateInContext: aBlock
	<omniDoNotTransform>
	^ OstDomain new evaluateInContext: aBlock! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/8/2011 21:53'!
readField: idx of: obj
	<omniDoNotTransform>
	^ obj instVarAt: idx! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 5/1/2012 21:27'!
readLiteral: association
	<omniDoNotTransform>
	^ association value! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 15:59'!
requestExecution: p1 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:19'!
requestExecution: p1 with: p2 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:24'!
requestExecution: p1 with: p2 with: p3 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:25'!
requestExecution: p1 with: p2 with: p3 with: p4 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/8/2011 21:53'!
requestExecutionOf: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/11/2011 20:52'!
write: val toField: idx of: obj
	<omniDoNotTransform>
	^ obj instVarAt: idx put: val.! !

!UndefinedObject methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 5/1/2012 21:26'!
write: val toLiteral: association
	<omniDoNotTransform>
	^ association value: val! !

!WriteStream methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:33'!
nextPut: anObject
	"Primitive. Insert the argument at the next position in the Stream
	represented by the receiver. Fail if the collection of this stream is not an
	Array or a String. Fail if the stream is positioned at its end, or if the
	position is out of bounds in the collection. Fail if the argument is not
	of the right type for the collection. Optional. See Object documentation
	whatIsAPrimitive."

	<primitive: 66>
	<omniReplacement: #atomicNextPut:>
	((collection class == ByteString) and: [
		anObject isCharacter and:[anObject isOctetCharacter not]]) ifTrue: [
			collection := (WideString from: collection).
			^self nextPut: anObject.
	].
	position >= writeLimit
		ifTrue: [^ self pastEndPut: anObject]
		ifFalse: 
			[position := position + 1.
			^collection at: position put: anObject]! !

!Array methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/9/2011 17:03'!
replaceFrom: start to: stop with: replacement startingAt: repStart
	"Primitive. This destructively replaces elements from start to stop in the receiver starting at index, repStart, in the collection, replacement. Answer the receiver. Range checks are performed in the primitive only. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 105>
	<omniReplacement: #omniReplaceFrom:to:with:startingAt:>
	super replaceFrom: start to: stop with: replacement startingAt: repStart! !

Array variableSubclass: #OstArray
	instanceVariableNames: 'omniDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Core-Misc'!

!OstArray class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/10/2012 13:41'!
fromArray: anArray
	| array index |
	array := self new: anArray size.
	index := 0.
	anArray do: [:each | array at: (index := index + 1) put: each].
	^ array! !

!OstArray class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/10/2012 13:26'!
initialize
	OstObject omnifyClass: OstArray! !

!OstArray class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/10/2012 14:17'!
new: size
	| arr |
	arr := super new: size.
	OstDomain initializeDomainForNewObject: arr of: self.
	^ arr! !

!OstArray methodsFor: 'omni reflection' stamp: ''!
omniDomain
	^ omniDomain! !

!OstArray methodsFor: 'omni reflection' stamp: ''!
omniDomain: t1 
	omniDomain := t1! !

!OstArray methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 
	<omniDoNotTransform>
	^ omniDomain requestExecutionOf: t1 on: self! !

!OstArray methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 lookup: t2 
	<omniDoNotTransform>
	^ omniDomain
		requestExecutionOf: t1
		lookup: t2
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:03'!
requestExecutionOf: aSel with: p1
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		of: aSel
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:22'!
requestExecutionOf: aSel with: p1 with: p2
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		of: aSel
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:32'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		of: aSel
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:33'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		of: aSel
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:35'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		of: aSel
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:37'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5 with: p6
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		of: aSel
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:38'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		of: aSel
		on: self! !

!OstArray methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 with: t2 with: t3 with: t4 with: t5 with: t6 with: t7 with: t8 with: t9 
	<omniDoNotTransform>
	^ omniDomain
		requestExecutionOf: t1
		on: self
		with: t2
		with: t3
		with: t4
		with: t5
		with: t6
		with: t7
		with: t8
		with: t9! !

!OstArray methodsFor: 'omni reflection' stamp: ''!
requestExecutionOf: t1 with: t2 with: t3 with: t4 with: t5 with: t6 with: t7 with: t8 with: t9 with: t10 
	<omniDoNotTransform>
	^ omniDomain
		requestExecutionOf: t1
		on: self
		with: t2
		with: t3
		with: t4
		with: t5
		with: t6
		with: t7
		with: t8
		with: t9
		with: t10! !

!OstArray methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:41'!
requestExecutionOf: aSel with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		with: p8 with: p9 with: p10
		of: aSel
		on: self! !

!RBLexicalScope methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/9/2011 15:20'!
numOmniTemps
	^ (bindings keys select: [:name | name beginsWith: 'x_omni_']) size! !

!RBOwnedScope methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/9/2011 15:20'!
numOmniTemps
	| parentCount |
	parentCount := owner ifNil:    [0]
	                     ifNotNil: [owner numOmniTemps].
	^ (bindings keys select: [:name | name beginsWith: 'x_omni_']) size + parentCount! !

!RBOwnedScope methodsFor: '*OmniST-Core-Compiler' stamp: 'StefanMarr 12/8/2011 15:19'!
theClass
	^ owner theClass! !

!MethodContext methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:35'!
tempAt: index 
	"Answer the value of the temporary variable whose index is the 
	 argument, index.  Primitive. Assumes receiver is indexable. Answer the
	 value of an indexable element in the receiver. Fail if the argument index
	 is not an Integer or is out of bounds. Essential. See Object documentation
	 whatIsAPrimitive.  Override the default at: primitive to give latitude to the
	 VM in context management."
	
	<omniDoNotTransform>
	<primitive: 210>
	^self at: index! !

!MethodContext methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:35'!
tempAt: index put: value 
	"Store the argument, value, as the temporary variable whose index is the 
	 argument, index.  Primitive. Assumes receiver is indexable. Answer the
	 value of an indexable element in the receiver. Fail if the argument index
	 is not an Integer or is out of bounds. Essential. See Object documentation
	 whatIsAPrimitive.  Override the default at:put: primitive to give latitude to
	 the VM in context management."

	<omniDoNotTransform>
	<primitive: 211>
	^self at: index put: value! !

!Integer methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:35'!
timesRepeat: aBlock
	"Evaluate the argument, aBlock, the number of times represented by the 
	receiver."

	<omniDoNotTransform>
	| count |
	count := 1.
	[count <= self]
		whileTrue: 
			[aBlock value.
			count := count + 1]! !

!RBVariableNode methodsFor: '*OmniST-Core-Compiler-ast-semantic-accessing' stamp: 'StefanMarr 12/4/2011 15:20'!
isInstanceBinding
	^ self variableBinding isInstanceBinding! !

!RBVariableNode methodsFor: '*OmniST-Core-Compiler-ast-semantic-accessing' stamp: 'StefanMarr 12/4/2011 15:27'!
isLiteralBinding
	^ self variableBinding isLiteralBinding! !

!RBVariableNode methodsFor: '*OmniST-Core-Compiler-ast-semantic-accessing' stamp: 'StefanMarr 12/4/2011 15:49'!
variableBinding: aBinding
	^ self propertyAt: #variableBinding put: aBinding! !

!SequenceableCollection methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/9/2011 17:04'!
omniReplaceFrom: start to: stop with: replacement startingAt: repStart
	| index repOff |
	repOff := repStart - start.
	index := start - 1.
	[ (index := index + 1) <= stop ]
		whileTrue: [ self at: index put: (replacement at: repOff + index) ]! !

!SharedQueue methodsFor: '*OmniST-Core-Reflection-accessing' stamp: 'StefanMarr 1/6/2012 15:37'!
peekWaiting
	^monitor critical: [
		monitor waitWhile: [ items isEmpty ].
		items first ]
! !

!BlockContext methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:15'!
tempAt: index 
	"Refer to the comment in ContextPart|tempAt:."

	<omniDoNotTransform>
	^ home at: index! !

!BlockContext methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:15'!
tempAt: index put: value 
	"Refer to the comment in ContextPart|tempAt:put:."

	<omniDoNotTransform>
	^ home at: index put: value! !

!Symbol methodsFor: '*OmniST-Core-Compiler-override' stamp: 'StefanMarr 12/11/2011 16:49'!
precedence
	"Answer the receiver's precedence, assuming it is a valid Smalltalk
	message selector or 0 otherwise.  The numbers are 1 for unary,
	2 for binary and 3 for keyword selectors."

	self isOmniSelector 
		ifTrue: [ ^ self asNormalSelector precedence ].
	self size = 0 ifTrue: [^ 0].
	self first isLetter ifFalse: [^ 2].
	self last = $: ifTrue: [^ 3].
	^ 1! !

RBCompiler subclass: #OstCompiler
	instanceVariableNames: 'doFullTransformation'
	classVariableNames: 'OldCompiler'
	poolDictionaries: ''
	category: 'OmniST-Core-Compiler'!

!OstCompiler class methodsFor: 'compiler support' stamp: 'StefanMarr 12/15/2011 14:13'!
disableCompiler
	OldCompiler = self ifTrue: [
		RBCompilerDispatcher current: RBCompiler.
	].
	RBCompilerDispatcher current: OldCompiler.! !

!OstCompiler class methodsFor: 'compiler support' stamp: 'StefanMarr 12/12/2011 00:52'!
enableCompiler
	| old |
	old := RBCompilerDispatcher current.
	
	"Remember old compiler only if it is not our own."
	old ~= self ifTrue: [
		OldCompiler := old].
	
	RBCompilerDispatcher current: self.! !

!OstCompiler class methodsFor: 'compiler support' stamp: 'StefanMarr 12/8/2011 14:54'!
flushAllOmniMethods
	Smalltalk allClassesAndTraits 
		do: [ :ea | ea flushOmniMethods. ea class flushOmniMethods ]
		displayingProgress: 'Flushing Omnified Methods'.! !

!OstCompiler class methodsFor: 'class initialization' stamp: 'StefanMarr 12/9/2011 16:41'!
initialize
	self enableCompiler.
	self registerChangeListener.
	
	(RBMessageNode hasInstVarNamed: #isOmniNode) ifFalse: [
		RBMessageNode addInstVarNamed: #isOmniNode.
	].! !

!OstCompiler class methodsFor: 'compiler support' stamp: 'StefanMarr 1/10/2012 14:21'!
installOmniVersionOf: selector in: class
	"Answer the atomic counterpart of aCompiledMethod.
	Methods with the pragma #omniDoNotTransform are considered to be already omnified.
	Methods with the pragma #omni: will use the annotation code for the omnified counterpart."
	<omniDoNotTransform>
	| omniCM omniName methDict compiledMethod |

	"Is already atomic do nothing"
	selector isOmniSelector ifTrue: [
		^ class methodDict at: selector ].

	omniName := selector asOmniSelector.
	methDict := class methodDict.
	compiledMethod := class methodDict at: selector.
		
	"These pragmas are specified in OstTransformer>>isOmniTransformationPragma:"
	compiledMethod pragmas do: [ :each |
		each keyword = #omniDoNotTransform
			ifTrue: [ ^ methDict at: omniName put: compiledMethod ].
			
		each keyword = #omniReplacement:
			ifTrue: [
				| replacementMeth |
				replacementMeth := class lookupSelector: each arguments first.
				omniCM := self installOmniVersionOf: each arguments first in: replacementMeth methodClass.
				^ methDict at: omniName put: omniCM.
			].
	].

	omniCM := self recompileOmnified: compiledMethod.
	^ methDict at: omniName put: omniCM! !

!OstCompiler class methodsFor: 'compiler support' stamp: 'StefanMarr 5/1/2012 22:30'!
omniMethod: aMessage missing: aReceiver in: aClass
	"This method is called whenever a missing atomic method is called. Installs a whole hierarchy of atomic methods."
	<omniDoNotTransform>
	
	| selector addedAtomicMethod curClass |
	selector := aMessage selector asNormalSelector.
	ProtoObject withAllSubclasses
		do: [ :class |
		(class methodDictionary includesKey: selector) ifTrue: [
			[self installOmniVersionOf: selector in: class.] value.
			[1 ifError: [:err :rcvr| Transcript cr; show: 'Error while handling a missing omni method. ',
									selector, ' did not compile properly for ', class asString,
									'. Compilation resulted in the following error: ',
									err].].
		]]
	displayingProgress: 'Omnify Methods: ', selector.
! !

!OstCompiler class methodsFor: 'events' stamp: 'StefanMarr 1/5/2012 16:35'!
onMethodModified: anObject
	<omniDoNotTransform>

	"Ignore the already omnified methods"
	anObject itemSelector isOmniSelector
		ifTrue: [^self].
	
	"Also blacklist a few things"
	((anObject itemClass = Debugger)
	or: [anObject itemClass = Decompiler])
		ifTrue: [^ self].

	"For the rest we do the atomic stuff"	
	self installOmniVersionOf: anObject itemSelector in:  anObject itemClass.! !

!OstCompiler class methodsFor: 'events' stamp: 'StefanMarr 12/11/2011 16:50'!
onMethodRemoved: anObject
	<omniDoNotTransform>
	
	anObject itemClass methodDict 
		removeKey: anObject itemSelector 
			asOmniSelector
		ifAbsent: nil! !

!OstCompiler class methodsFor: 'compiling' stamp: 'StefanMarr 12/7/2011 22:24'!
recompile: aCompiledMethod
	"Recompile the given method normally"
	<omniDoNotTransform>
	
	| compiler |
	compiler := self new.
	^ compiler recompile: aCompiledMethod.! !

!OstCompiler class methodsFor: 'compiling' stamp: 'StefanMarr 12/7/2011 22:24'!
recompileOmnified: aCompiledMethod
	"Recompile the given method normally"
	<omniDoNotTransform>
	
	| compiler |
	compiler := self new.
	^ compiler recompileOmnified: aCompiledMethod.! !

!OstCompiler class methodsFor: 'compiler support' stamp: 'StefanMarr 1/2/2012 18:47'!
recompilePackageWithOstCompiler
	<omniDoNotTransform>
	
	self enableCompiler.
	self flushAllOmniMethods.

	(PackageInfo named: 'OmniST') classes
		do: [:c | c compileAll]
		displayingProgress: 'Recompiling OmniST with OstCompiler support enabled'.! !

!OstCompiler class methodsFor: 'class initialization' stamp: 'StefanMarr 12/8/2011 14:40'!
registerChangeListener

	SystemChangeNotifier uniqueInstance
		"Currently I only do lazy compilation on first use for new stuff"
		"notify: self ofSystemChangesOfItem: #method change: #Added using: #onMethodAdded:;"
		
		"For existing stuff, however, we need to ensure consistency"
		notify: self ofSystemChangesOfItem: #method change: #Modified using: #onMethodModified:;
		notify: self ofSystemChangesOfItem: #method change: #Removed using: #onMethodRemoved:.! !

!OstCompiler class methodsFor: 'class initialization' stamp: 'StefanMarr 12/9/2011 16:42'!
unload
	self flushAllOmniMethods.
	
	RBMessageNode removeInstVarNamed: #isOmniNode.! !

!OstCompiler class methodsFor: 'class initialization' stamp: 'StefanMarr 12/7/2011 22:11'!
unregisterChangeListener

	SystemChangeNotifier uniqueInstance
		noMoreNotificationsFor: self.! !

!OstCompiler methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 22:11'!
initialize
	super initialize.
	doFullTransformation := false.! !

!OstCompiler methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 22:26'!
parse: methodBoolean
	| methodNode transformer |
	methodNode := super parse: methodBoolean.
	
	methodBoolean ifTrue: [
		transformer := OstTransformer new.
		transformer fullTransformation: doFullTransformation.
		transformer applyTransformations: methodNode inScopeOf: class.
	].
	^ methodNode! !

!OstCompiler methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 22:14'!
recompile: aCompiledMethod
	"Recompile the given method normally, i.e., with out full STM transformation"

	^ self recompile: aCompiledMethod omnified: false.! !

!OstCompiler methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/11/2011 16:50'!
recompile: aCompiledMethod omnified: aBool
	"Recompile the given method"

	| methodSource newSelector  newMethodNode |
	
	doFullTransformation := aBool.
	
	methodSource := aCompiledMethod getSource.
	newSelector  := aCompiledMethod selector asOmniSelector.
		
	newMethodNode := self
		compile: methodSource in: aCompiledMethod methodClass notifying: self
		ifFail: [^ self problem: 'syntax error' sel: newSelector in: aCompiledMethod methodClass].

	^ newMethodNode generate.! !

!OstCompiler methodsFor: 'as yet unclassified' stamp: 'StefanMarr 12/7/2011 22:14'!
recompileOmnified: aCompiledMethod
	"Recompile the given method with the STM enabled"

	^ self recompile: aCompiledMethod omnified: true.! !

Object subclass: #OPromise
	instanceVariableNames: 'signal value'
	classVariableNames: 'NoValueDelivered'
	poolDictionaries: ''
	category: 'OmniST-Core-Misc'!

!OPromise class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 11/6/2011 00:19'!
example
	"This is not a real test case, but just to see how it is supposed to work"
	| p |
	p := OPromise new.
	
	[|val| 
	 Transcript log: 'About to read promised value.'.
	 val := p read.
	 Transcript log: 'Val: ', val asString.] fork.

	Processor yield.
	
	Transcript log: 'About to deliver promised value.'.
	p deliver: 42.
	Transcript log: 'Deliver promised value.'.
	! !

!OPromise class methodsFor: 'initialization' stamp: 'StefanMarr 11/4/2011 15:53'!
initialize
	NoValueDelivered := Object new.! !

!OPromise methodsFor: 'promise' stamp: 'StefanMarr 1/9/2012 17:53'!
await
	<omniDoNotTransform>
	[value = NoValueDelivered] whileTrue: [
		"This is ugly, but I do not see how I can prevent the data
		 race for signals otherwise.
		 In case there are multiple possible consumers everyone
		 but the first will starve when they arrive before the
		 promise was delivered.
		 Now, we just have a delay, but it is guaranteed to return."
		signal waitTimeoutMSecs: 100.
	].! !

!OPromise methodsFor: 'promise' stamp: 'StefanMarr 1/9/2012 17:53'!
deliver: aValue
	<omniDoNotTransform>
	"Deliver the promised value."
	
	
	"Warning: this is racy, but we don't care for the moment.
	 Should be a single produce anyway. Everything else is a bug.
	 And, we do at least best effort error notification..."
	value = NoValueDelivered ifFalse: [
		| err |
		err := Error new.
		err messageText: 'Promise has been delivered more than once.'.
		err signal.
	].

	value := aValue.
	
	signal signal.
	
	^ value.! !

!OPromise methodsFor: 'initialization' stamp: 'StefanMarr 11/4/2011 16:09'!
initialize
	value := NoValueDelivered.
	signal := Semaphore new.! !

!OPromise methodsFor: 'accessing' stamp: 'StefanMarr 1/9/2012 17:53'!
read
	<omniDoNotTransform>
	self await.
	^ value! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/3/2012 15:44'!
at: index 
	"Primitive. Assumes receiver is indexable. Answer the value of an 
	indexable element in the receiver. Fail if the argument index is not an 
	Integer or is out of bounds. Essential. See Object documentation 
	whatIsAPrimitive."

	<primitive: 60>
	<omniReplacement: #omniAt:>
	index isInteger ifTrue:
		[self class isVariable
			ifTrue: [self errorSubscriptBounds: index]
			ifFalse: [self errorNotIndexable]].
	index isNumber
		ifTrue: [^self at: index asInteger]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/3/2012 15:49'!
at: index put: value
	"Primitive. Assumes receiver is indexable. Store the argument value in 
	the indexable element of the receiver indicated by index. Fail if the 
	index is not an Integer or is out of bounds. Or fail if the value is not of 
	the right type for this kind of collection. Answer the value that was 
	stored. Essential. See Object documentation whatIsAPrimitive."

	<primitive: 61>
	<omniReplacement: #omniAt:put:>
	index isInteger ifTrue:
		[self class isVariable
			ifTrue: [(index >= 1 and: [index <= self size])
					ifTrue: [self errorImproperStore]
					ifFalse: [self errorSubscriptBounds: index]]
			ifFalse: [self errorNotIndexable]].
	index isNumber
		ifTrue: [^self at: index asInteger put: value]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 21:57'!
baselevelPerform: aSymbol 
	<primitive: 83>
	^ self perform: aSymbol withArguments: (Array new: 0)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 20:51'!
baselevelPerform: aSymbol with: anObject 
	<primitive: 83>
	^ self perform: aSymbol withArguments: (Array with: anObject)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 20:52'!
baselevelPerform: aSymbol with: firstObject with: secondObject 
	<primitive: 83>
	^ self perform: aSymbol withArguments: (Array with: firstObject with: secondObject)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 20:52'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject 
	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:00'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:01'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4 with: o5)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:01'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:01'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7)! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:02'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7 with: o8
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: {firstObject. secondObject. thirdObject. o4. o5. o6. o7. o8}! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:02'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7 with: o8 with: o9
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: {firstObject. secondObject. thirdObject. o4. o5. o6. o7. o8. o9}! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:02'!
baselevelPerform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7 with: o8 with: o9 with: o10
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: {firstObject. secondObject. thirdObject. o4. o5. o6. o7. o8. o9. o10}! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 20:49'!
baselevelPerform: selector withArguments: argArray 
	<primitive: 84>
	^ self perform: selector withArguments: argArray inSuperclass: self class! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 22:00'!
baselevelPerform: selector withArguments: argArray inSuperclass: lookupClass
	"NOTE:  This is just like perform:withArguments:, except that
	the message lookup process begins, not with the receivers's class,
	but with the supplied superclass instead.  It will fail if lookupClass
	cannot be found among the receiver's superclasses.
	Primitive. Essential. See Object documentation whatIsAPrimitive."

	<primitive: 100>
	(selector isSymbol)
		ifFalse: [^ self error: 'selector argument must be a Symbol'].
	(selector numArgs = argArray size)
		ifFalse: [^ self error: 'incorrect number of arguments'].
	(self class == lookupClass or: [self class inheritsFrom: lookupClass])
		ifFalse: [^ self error: 'lookupClass is not in my inheritance chain'].
	self primitiveFailed! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 21:59'!
baselevelPerformSuper: aSymbol inLookupClass: cls
	<primitive: 100>
	^ self perform: aSymbol withArguments: (Array new: 0) inSuperclass: cls! !

!Object methodsFor: '*OmniST-Core-VM-Compat' stamp: 'StefanMarr 5/1/2012 21:58'!
baselevelPerformSuper: aSymbol with: args inLookupClass: cls
	<primitive: 100>
	^ self perform: aSymbol withArguments: args inSuperclass: cls! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/3/2012 15:51'!
basicAt: index
	"Primitive. Assumes receiver is indexable. Answer the value of an 
	indexable element in the receiver. Fail if the argument index is not an 
	Integer or is out of bounds. Essential. Do not override in a subclass. See 
	Object documentation whatIsAPrimitive."

	<primitive: 60>
	<omniReplacement: #omniBasicAt:>
	index isInteger ifTrue: [self errorSubscriptBounds: index].
	index isNumber
		ifTrue: [^self basicAt: index asInteger]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/3/2012 15:51'!
basicAt: index put: value
	"Primitive. Assumes receiver is indexable. Store the second argument 
	value in the indexable element of the receiver indicated by index. Fail 
	if the index is not an Integer or is out of bounds. Or fail if the value is 
	not of the right type for this kind of collection. Answer the value that 
	was stored. Essential. Do not override in a subclass. See Object 
	documentation whatIsAPrimitive."

	<primitive: 61>
	<omniReplacement: #omniBasicAt:put:>
	index isInteger
		ifTrue: [(index >= 1 and: [index <= self size])
					ifTrue: [self errorImproperStore]
					ifFalse: [self errorSubscriptBounds: index]].
	index isNumber
		ifTrue: [^self basicAt: index asInteger put: value]
		ifFalse: [self errorNonIntegerIndex]! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/5/2012 16:45'!
doesNotUnderstand: aMessage 
	 "Handle the fact that there was an attempt to send the given message to the receiver but the receiver does not understand this message (typically sent from the machine when a message is sent to the receiver and no method is defined for that selector)."
	"Testing: (3 activeProcess)"
	"fixed suggested by Eliot miranda to make sure 
	
	[Object new blah + 1]
		on: MessageNotUnderstood
		do: [:e | e resume: 1] does not loop indefinitively"
		
	| exception resumeValue |
	
	<omniDoNotTransform>	
	aMessage selector isOmniSelector
		ifTrue: [
			OstCompiler omniMethod: aMessage missing: self in: aMessage lookupClass.
			
			((self respondsTo: aMessage selector) or: [
			  self isBehavior and: [self canUnderstand: aMessage selector]])
			ifTrue: [
				"And try again."
				^ aMessage sentTo: self.
			]].
		
	(exception := MessageNotUnderstood new)
				message: aMessage;
				receiver: self.
	resumeValue := exception signal.
	^exception reachedDefaultHandler
				ifTrue: [aMessage sentTo: self]
				ifFalse: [resumeValue]! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 12/11/2011 16:25'!
halt
	"This is the typical message to use for inserting breakpoints during 
	debugging. It behaves like halt:, but does not call on halt: in order to 
	avoid putting this message on the stack. Halt is especially useful when 
	the breakpoint message is an arbitrary one."

	<omniDoNotTransform>
	Halt signal! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/3/2012 15:59'!
instVarAt: index 
	"Primitive. Answer a fixed variable in an object. The numbering of the 
	variables corresponds to the named instance variables. Fail if the index 
	is not an Integer or is not the index of a fixed variable. Essential. See 
	Object documentation whatIsAPrimitive."

	<primitive: 73>
	<omniReplacement: #omniInstVarAt:>
	"Access beyond fixed variables."
	^self basicAt: index - self class instSize		! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/3/2012 16:01'!
instVarAt: anInteger put: anObject 
	"Primitive. Store a value into a fixed variable in the receiver. The 
	numbering of the variables corresponds to the named instance variables. 
	Fail if the index is not an Integer or is not the index of a fixed variable. 
	Answer the value stored as the result. Using this message violates the 
	principle that each object has sovereign control over the storing of 
	values into its instance variables. Essential. See Object documentation 
	whatIsAPrimitive."

	<primitive: 74>
	<omniReplacement: #omniInstVarAt:put:>
	"Access beyond fixed fields"
	^self basicAt: anInteger - self class instSize put: anObject! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/3/2012 15:58'!
omniAt: idx
	<omniDoNotTransform>
	^ OstDomain currentDomain primAt: idx on: self.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/3/2012 15:58'!
omniAt: idx put: anObject
	<omniDoNotTransform>
	^ OstDomain currentDomain primAt: idx on: self put: anObject.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/3/2012 15:58'!
omniBasicAt: idx
	<omniDoNotTransform>
	^ OstDomain currentDomain primBasicAt: idx on: self.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/3/2012 15:58'!
omniBasicAt: idx put: anObject
	<omniDoNotTransform>
	^ OstDomain currentDomain primBasicAt: idx on: self put: anObject.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/9/2012 21:54'!
omniDomain
	^ nil! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/9/2012 21:54'!
omniDomain: aDomain
	^ nil! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/3/2012 15:58'!
omniInstVarAt: anInteger
	<omniDoNotTransform>
	^ OstDomain currentDomain primInstVarAt: anInteger on: self.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/3/2012 16:01'!
omniInstVarAt: anInteger put: aVal
	<omniDoNotTransform>
	^ OstDomain currentDomain primInstVarAt: anInteger on: self put: aVal.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/10/2012 11:41'!
omniShallowCopy
	<omniDoNotTransform>
	^ OstDomain currentDomain primShallowCopy: self.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/9/2011 15:59'!
perform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4)! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/12/2011 16:40'!
perform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4 with: o5)! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/12/2011 16:40'!
perform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6)! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/12/2011 16:40'!
perform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: (Array with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7)! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/8/2012 14:51'!
perform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7 with: o8
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: {firstObject. secondObject. thirdObject. o4. o5. o6. o7. o8}! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/8/2012 14:51'!
perform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7 with: o8 with: o9
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: {firstObject. secondObject. thirdObject. o4. o5. o6. o7. o8. o9}! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/8/2012 14:51'!
perform: aSymbol with: firstObject with: secondObject with: thirdObject with: o4 with: o5 with: o6 with: o7 with: o8 with: o9 with: o10
	"Send the selector, aSymbol, to the receiver with the given arguments.
	Fail if the number of arguments expected by the selector is not three.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 83>
	^ self perform: aSymbol
		withArguments: {firstObject. secondObject. thirdObject. o4. o5. o6. o7. o8. o9. o10}! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/12/2011 09:38'!
performSuper: aSymbol inLookupClass: cls
	<primitive: 100>
	^ self perform: aSymbol withArguments: (Array new: 0) inSuperclass: cls! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 1/5/2012 16:08'!
performSuper: aSymbol with: args inLookupClass: cls
	"Send the selector, aSymbol, to the receiver with anObject as its argument.
	Fail if the number of arguments expected by the selector is not one.
	Primitive. Optional. See Object documentation whatIsAPrimitive."

	<primitive: 100>
	^ self perform: aSymbol withArguments: args inSuperclass: cls! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/15/2011 14:06'!
requestExecutionOf: aSelector 
	^ OstDomain currentDomain requestExecutionOf: aSelector on: self.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/15/2011 16:00'!
requestExecutionOf: aSelector lookup: cls
	^ OstDomain currentDomain requestExecutionOf: aSelector lookup: cls on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 12/15/2011 15:07'!
requestExecutionOf: aSelector lookup: cls with: p1
	^ OstDomain currentDomain requestExecutionOf: aSelector lookup: cls on: self with: p1.! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:02'!
requestExecutionOf: aSelector with: p1
	^ OstDomain currentDomain requestExecution: p1 of: aSelector on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:22'!
requestExecutionOf: aSelector with: p1 with: p2
	^ OstDomain currentDomain requestExecution: p1 with: p2 of: aSelector on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:32'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3
	^ OstDomain currentDomain 
	requestExecution: p1
		with: p2
		with: p3
		of: aSelector
		on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:34'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4
	^ OstDomain currentDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		of: aSelector
		on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:36'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5
	^ OstDomain currentDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		of: aSelector
		on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:37'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6
	^ OstDomain currentDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		of: aSelector
		on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:38'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7
	^ OstDomain currentDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		of: aSelector 
		on: self! !

!Object methodsFor: '*OmniST-Core-Reflection' stamp: 'StefanMarr 3/22/2012 16:41'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10
	^ OstDomain currentDomain 
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		with: p8 with: p9 with: p10
		of: aSelector 
		on: self! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/10/2012 11:42'!
shallowCopy
	"Answer a copy of the receiver which shares the receiver's instance variables. It should never be overridden. I'm invoked from the copy template method. Subclasses that need to specialize the copy should specialize the postCopy hook method."
	
	| class newObject index |
	<omniReplacement: #omniShallowCopy>
	<primitive: 148>
	class := self class.
	class isVariable
		ifTrue: 
			[index := self basicSize.
			newObject := class basicNew: index.
			[index > 0]
				whileTrue: 
					[newObject basicAt: index put: (self basicAt: index).
					index := index - 1]]
		ifFalse: [newObject := class basicNew].
	index := class instSize.
	[index > 0]
		whileTrue: 
			[newObject instVarAt: index put: (self instVarAt: index).
			index := index - 1].
	^ newObject! !

!Object methodsFor: '*OmniST-Core-Reflection-overrides' stamp: 'StefanMarr 1/5/2012 20:22'!
subclassResponsibility
	"This message sets up a framework for the behavior of the class' subclasses.
	Announce that the subclass should have implemented this message."
	<omniDoNotTransform>
	SubclassResponsibility signalFor: thisContext sender selector! !

Object subclass: #OstDomain
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Core-Reflection'!

!OstDomain class methodsFor: 'omni reflection' stamp: 'StefanMarr 12/8/2011 16:36'!
currentDomain
	^ Processor activeProcess omniDomain! !

!OstDomain class methodsFor: 'omni reflection' stamp: 'StefanMarr 12/7/2011 23:08'!
domainIndexFromClass: cls
	^ cls instVarIndexFor: #omniDomain! !

!OstDomain class methodsFor: 'omni reflection' stamp: 'StefanMarr 12/11/2011 18:32'!
domainIndexFromObject: obj
	^ self domainIndexFromClass: obj class! !

!OstDomain class methodsFor: 'class initialization' stamp: 'StefanMarr 12/8/2011 16:12'!
initialize
	(Process hasInstVarNamed: #omniDomain) ifFalse: [
		Process addInstVarNamed: #omniDomain.
	].! !

!OstDomain class methodsFor: 'omni instantiation' stamp: 'StefanMarr 3/9/2012 21:56'!
initializeDomainForNewObject: obj of: class
	"We rely here currently on the getters/setters being available.
	 This is not exactly in sync with what the 'Mirrorified' reflection API for Omni actually should provide.
	 but I hope this does not have to many performance implications. :("
	obj omniDomain: self currentDomain domainForNewObjects.
! !

!OstDomain class methodsFor: 'omni reflection' stamp: 'StefanMarr 12/11/2011 18:37'!
omniDomainFrom: obj
	| idx |
	idx := self domainIndexFromObject: obj.
	
	idx ~= 0 ifTrue: [
		^ obj instVarAt: idx.
	].

	^ nil! !

!OstDomain class methodsFor: 'omni reflection' stamp: 'StefanMarr 1/9/2012 23:56'!
setOmniDomainOn: obj of: cls to: aDomain
	| idx |
	idx := self domainIndexFromClass: cls.
	idx > 0 ifTrue: [
		obj instVarAt: idx put: aDomain.].! !

!OstDomain class methodsFor: 'initialize-release' stamp: 'StefanMarr 1/9/2012 22:53'!
unload
	(self == OstDomain and: [Process hasInstVarNamed: #omniDomain]) ifTrue: [
		Process removeInstVarNamed: #omniDomain.
	].! !

!OstDomain methodsFor: 'omni-domain' stamp: 'StefanMarr 1/2/2012 17:56'!
adoptObject: obj
	self class setOmniDomainOn: obj of: obj class to: self.
	^ obj! !

!OstDomain methodsFor: 'omni-domain' stamp: 'StefanMarr 12/8/2011 14:06'!
domainForNewObjects
	"#basicNew need to determine which domain a new object is supposed
	 to belong to.
	 The standard domain is self, the domain of the currently executing
	 OProcess.
	
	 If this method is overriden, make sure that you do not allocate any objects here!!!!!!"
	^ self! !

!OstDomain methodsFor: 'omni-domain' stamp: 'StefanMarr 12/8/2011 16:37'!
evaluateInContext: aBlock
	| oldDomain proc result |
	proc := Processor activeProcess.
	
	oldDomain := proc omniDomain.
	
	proc omniDomain: self.
	
	result := aBlock value.
	
	proc omniDomain: oldDomain.
	
	^ result! !

!OstDomain methodsFor: 'omni-domain' stamp: 'StefanMarr 12/11/2011 20:05'!
newProcessFor: aBlock
	"Answer a Process running the code in aBlock in this domain.
	 The process is not scheduled."
	| ctx proc |
	<primitive: 19> "Simulation guard"
	ctx := [aBlock value.
			Processor terminateActive] asContext.
	proc := Process
		forContext: ctx
		priority: Processor activePriority.
	proc omniDomain: self.
	^ proc.! !

!OstDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 1/3/2012 16:13'!
primAt: idx on: anObj
	<omniDoNotTransform>
	^ anObj at: idx! !

!OstDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 1/3/2012 16:13'!
primAt: idx on: anObj put: aVal
	<omniDoNotTransform>
	^ anObj at: idx put: aVal! !

!OstDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 1/3/2012 16:13'!
primBasicAt: idx on: anObj
	<omniDoNotTransform>
	^ anObj basicAt: idx! !

!OstDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 1/3/2012 16:14'!
primBasicAt: idx on: anObj put: aVal
	<omniDoNotTransform>
	^ anObj basicAt: idx put: aVal! !

!OstDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 1/3/2012 16:14'!
primInstVarAt: idx on: anObj
	<omniDoNotTransform>
	^ anObj instVarAt: idx! !

!OstDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 1/3/2012 16:14'!
primInstVarAt: idx on: anObj put: aVal
	<omniDoNotTransform>
	^ anObj instVarAt: idx put: aVal! !

!OstDomain methodsFor: 'omni primitive handlers' stamp: 'StefanMarr 1/10/2012 11:40'!
primShallowCopy: anObj
	<omniDoNotTransform>
	^ anObj shallowCopy! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 12/15/2011 15:03'!
readField: idx of: obj
	<omniDoNotTransform>
	^ obj instVarAt: idx! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 5/1/2012 21:33'!
readLiteral: association
	<omniDoNotTransform>
	^ association value! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 15:57'!
requestExecution: p1 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 16:18'!
requestExecution: p1 with: p2 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 16:24'!
requestExecution: p1 with: p2 with: p3 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 16:25'!
requestExecution: p1 with: p2 with: p3 with: p4 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 16:26'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 16:27'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 16:28'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 3/22/2012 16:29'!
requestExecution: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10 of: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 12/12/2011 17:53'!
requestExecutionOf: aSelector lookup: cls on: anObject
	<omniDoNotTransform>
	^ anObject performSuper: aSelector inLookupClass: cls! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 1/5/2012 16:09'!
requestExecutionOf: aSelector lookup: cls on: anObject with: p1
	<omniDoNotTransform>
	^ anObject performSuper: aSelector with: { p1 } inLookupClass: cls.! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 12/11/2011 22:41'!
requestExecutionOf: aSelector on: anObject
	<omniDoNotTransform>
	^ anObject perform: aSelector! !

!OstDomain methodsFor: 'omni-domain' stamp: 'StefanMarr 12/8/2011 14:07'!
spawnHere: aBlock
	"Spawn the given block as a process in this domain"
	
	self flag: 'WARNING: copied from ODomain.
	TODO: fix later after abstracting concepts from the results of the experiments.'.
	
	^ (self newProcessFor: aBlock) resume! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 12/15/2011 15:02'!
write: val toField: idx of: obj
	<omniDoNotTransform>
	^ obj instVarAt: idx put: val.! !

!OstDomain methodsFor: 'omni handlers' stamp: 'StefanMarr 5/1/2012 21:33'!
write: val toLiteral: association
	<omniDoNotTransform>
	^ association value: val! !

Object subclass: #OstMirror
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Core-Reflection'!

!OstMirror class methodsFor: 'primitives' stamp: 'StefanMarr 3/20/2012 15:48'!
domainOf: obj
	^ obj omniDomain! !

!OstMirror class methodsFor: 'primitives' stamp: 'StefanMarr 5/1/2012 22:42'!
executesOnBaseLevel
	<omniDoNotTransform>
	thisContext sender ifNotNilDo: [:senderCtx |
		(senderCtx selector beginsWith: #request) ifTrue: [
			| args |
			args := senderCtx arguments.
			^ (args at: (args size - 1)) isOmniSelector.
		]
	].
	
	^ false.! !

!OstMirror class methodsFor: 'primitives' stamp: 'StefanMarr 3/20/2012 15:49'!
setDomainOf: obj to: domain
	obj omniDomain: domain! !

Object subclass: #OstObject
	instanceVariableNames: 'omniDomain'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'OmniST-Core-Misc'!

!OstObject class methodsFor: 'initialization' stamp: 'StefanMarr 1/6/2012 00:56'!
initialize
	"Here we will need something to bring our OstDomain like properties, for instance as a trait to at least some of the stream classes"! !

!OstObject class methodsFor: 'omnify' stamp: 'StefanMarr 1/10/2012 13:27'!
omnifyClass: aClass
	"First add all inst vars"
	self instVarNames do: [:varName | 
		(aClass hasInstVarNamed: varName) ifFalse: [
			aClass addInstVarNamed: varName]].
	
	"Then copy the methods"
	(self organization listAtCategoryNamed: 'omni reflection') do: [:selector |
		| method |
		method := self lookupSelector: selector.
		aClass compile: method getSource
		       classified: method category
		       withStamp: method timeStamp
		       notifying: nil
		       logSource: false.
	].! !

!OstObject class methodsFor: 'initialize-release' stamp: 'StefanMarr 1/4/2012 15:22'!
unload
	"Revert changes"
	
	Stream superclass: Object.! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 1/3/2012 14:04'!
omniDomain
	^ omniDomain! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 1/3/2012 14:04'!
omniDomain: aDomain
	omniDomain := aDomain! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 1/3/2012 09:42'!
requestExecutionOf: aSelector
	<omniDoNotTransform>
	^ omniDomain requestExecutionOf: aSelector on: self.! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 1/3/2012 09:44'!
requestExecutionOf: aSelector lookup: cls
	<omniDoNotTransform>
	^ omniDomain requestExecutionOf: aSelector lookup: cls on: self.! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:04'!
requestExecutionOf: aSelector with: p1
	<omniDoNotTransform>
	^ omniDomain requestExecution: p1 of: aSelector on: self! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:19'!
requestExecutionOf: aSelector with: p1 with: p2
	<omniDoNotTransform>
	^ omniDomain requestExecution: p1 with: p2 of: aSelector on: self! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:32'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		of: aSelector
		on: self! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:34'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		of: aSelector
		on: self! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:35'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		of: aSelector 
		on: self! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:38'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		of: aSelector
		on: self! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:39'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		of: aSelector
		on: self! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 1/3/2012 09:42'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8
	<omniDoNotTransform>
	^ omniDomain requestExecutionOf: aSelector on: self with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 1/3/2012 09:43'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9
	<omniDoNotTransform>
	^ omniDomain requestExecutionOf: aSelector on: self with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9! !

!OstObject methodsFor: 'omni reflection' stamp: 'StefanMarr 3/22/2012 16:41'!
requestExecutionOf: aSelector with: p1 with: p2 with: p3 with: p4 with: p5 with: p6 with: p7 with: p8 with: p9 with: p10
	<omniDoNotTransform>
	^ omniDomain
		requestExecution: p1
		with: p2
		with: p3
		with: p4
		with: p5
		with: p6
		with: p7
		with: p8 with: p9 with: p10
		of: aSelector
		on: self! !

!ReadStream methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:33'!
next
	"Primitive. Answer the next object in the Stream represented by the
	receiver. Fail if the collection of this stream is not an Array or a String.
	Fail if the stream is positioned at its end, or if the position is out of
	bounds in the collection. Optional. See Object documentation
	whatIsAPrimitive."

	<primitive: 65>
	<omniReplacement: #atomicNext>
	position >= readLimit
		ifTrue: [^nil]
		ifFalse: [^collection at: (position := position + 1)]! !

!ContextPart methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:18'!
canHandleSignal: exception
	"Sent to handler (on:do:) contexts only.  If my exception class (first arg) handles exception then return true, otherwise forward this message to the next handler context.  If none left, return false (see nil>>canHandleSignal:)"

	<omniDoNotTransform>
	^ (((self tempAt: 1) handles: exception) and: [self tempAt: 3])
		or: [self nextHandlerContext canHandleSignal: exception].
! !

!ContextPart methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 16:18'!
handleSignal: exception
	"Sent to handler (on:do:) contexts only.  If my exception class (first arg) handles exception then execute my handle block (second arg), otherwise forward this message to the next handler context.  If none left, execute exception's defaultAction (see nil>>handleSignal:)."

	<omniDoNotTransform>
	| val |
	(((self tempAt: 1) handles: exception) and: [self tempAt: 3]) ifFalse: [
		^ self nextHandlerContext handleSignal: exception].

	exception privHandlerContext: self contextTag.
	self tempAt: 3 put: false.  "disable self while executing handle block"
	val := [(self tempAt: 2) cull: exception]
		ensure: [self tempAt: 3 put: true].
	self return: val.  "return from self if not otherwise directed in handle block"
! !

!String methodsFor: '*OmniST-Core-Compiler-overrides' stamp: 'StefanMarr 12/8/2011 21:56'!
numArgs 
	"Answer either the number of arguments that the receiver would take if considered a selector.  Answer -1 if it couldn't be a selector. It is intended mostly for the assistance of spelling correction."

	| firstChar numColons start ix |
	self size = 0 ifTrue: [^ -1].
	self isOmniSelector 
		ifTrue: [ ^ self asNormalSelector numArgs ].
	firstChar := self at: 1.
	(firstChar isLetter or: [ firstChar = $_]) ifTrue:
		["Fast reject if any chars are non-alphanumeric
		NOTE: fast only for Byte things - Broken for Wide"
		self class isBytes
			ifTrue: [(self findSubstring: '~' in: self startingAt: 1 matchTable: Tokenish) > 0 ifTrue: [^ -1]]
			ifFalse: [2 to: self size do: [:i | (self at: i) tokenish ifFalse: [^ -1]]].
		"Fast colon count"
		numColons := 0.  start := 1.
		[(ix := self indexOf: $: startingAt: start) > 0]
			whileTrue:
				[ix = start ifTrue: [^-1].
				numColons := numColons + 1.
				start := ix + 1].
		numColons = 0 ifTrue: [^ 0].
		self last = $:
			ifTrue: [^ numColons]
			ifFalse: [^ -1]].
	firstChar isSpecial ifTrue:
		[self size = 1 ifTrue: [^ 1].
		2 to: self size do: [:i | (self at: i) isSpecial ifFalse: [^ -1]].
		^ 1].
	^ -1.! !

!String methodsFor: '*OmniST-Core-Reflection-override' stamp: 'StefanMarr 12/11/2011 18:48'!
replaceFrom: start to: stop with: replacement startingAt: repStart
	"Primitive. This destructively replaces elements from start to stop in the receiver starting at index, repStart, in the collection, replacement. Answer the receiver. Range checks are performed in the primitive only. Optional. See Object documentation whatIsAPrimitive."
	
	<primitive: 105>
	<omniReplacement: #omniReplaceFrom:to:with:startingAt:>
	super replaceFrom: start to: stop with: replacement startingAt: repStart! !
OstTransformer initialize!
OPromise initialize!
OstObject initialize!
OstArray initialize!
OstCompiler initialize!
OstDomain initialize!
